import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category rds service config
 */
const currentRDSServiceConfig = globalValue("@effect-aws/client-rds/currentRDSServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category rds service config
 */
export const withRDSServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentRDSServiceConfig, config));
/**
 * @since 1.0.0
 * @category rds service config
 */
export const setRDSServiceConfig = (config) => Layer.locallyScoped(currentRDSServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toRDSClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentRDSServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=RDSServiceConfig.js.map